%% solution to the exercise of nkmodel_43.mod

%% exercise 1: linearly detrend the data before estimation
%% exercise 2: make the priors less informatives, compare results
%% exercise 3: add measurement error to the model (i.e the data is just a proxy for the data simualted from the model)
%% exercise 4: use a forward looking taylor tule
%% exercise 5: assume that shocks are correlated

// NEW KEYNESIAN MODEL, version restricted for  subsamples
// IS:  x(t) = E_t{x(t+1)} - 1/phi*( r(t) - E_t{p(t+1)} ) + g(t)
// PC:  p(t) = betta*E_t{p(t+1)} + kappa*x(t) + u(t)
// Taylor-Rule : r(t) = psir*r(t-1) + (1-psir)*(phix*p(t) + psip*p(t)) + er(t)
//
//  E(t)=(g(t), u(t), er(t))
//  E(t) = rho*E(t-1) + A*ee(t);  ee ~ nid(0,sigg^2),  where A is a matrix
//  and rho = diag(rhog, rhou, 0)
//  p = inflation, x = output gap, r = interest rate
//
// kappa = lambda * (phi+nu)( 1-theta)*(1-betta*theta)/theta


% preliminary data management
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load gpr.dat;              % data from Ireland (2002) - downloaded 2013 (LER)
[nobs,nvar] = size(gpr);   % # Observations in time series
y = gpr(:,1);
pp = gpr(:,2);
rr = gpr(:,3);

%******L********L**********L*****************L********L**********L***********
%   This section will be ommited in the following release.
%   The bridge methodology will account for the trends directly.
%******L********L**********L*****************L********L**********L***********
//dates = cal(1948,1,4);
// detrend y, remove linear trend
cr = ones(nobs,1);
tr = (1:nobs)';
for i=1:nobs
tr2(i) = tr(i)^2;
end;
yt = FOLS(y,[cr tr tr2']);
x = y-yt.beta(2)*tr-yt.beta(3)*tr2';       % output gap
pt = FOLS(pp,[cr tr tr2']);
p = pp-pt.beta(1)-pt.beta(2)*tr-pt.beta(3)*tr2';       % eliminate quadratic trend
rt = FOLS(rr,[cr tr tr2']);
r = rr-rt.beta(1)-rt.beta(2)*tr-rt.beta(3)*tr2';       % eliminate quadratic trend

 
save detdataUS_2  rr xx pp;
  
parameters phi betta kappa psir psix psip rhog rhou sigg sigu sigr;

% for exercise 5
%parameters phi betta kappa psir psix psip rhog rhou sigg sigu sigr a12 a13;

phi= 2.0;
betta= 0.98;
kappa= 2.00;
psir= 0.98;
psix= 0.03;
psip= 2.00;
rhog= 0.85;
rhou= 0.85;
a12 = 0.1;
a13 = 0.1;

%var r x p g u eer;    
%varexo eg eu er;  

% for exercise 3:
 var r x p g u eer rr xx pp;    
 varexo eg eu er u1 u2 u3;  

%model(linear); 
%           x = x(1)-(1/phi)*(r-p(1))+g;
%           p = betta*p(1)+kappa*x+u;
%           r = psir*r(-1)+(1-psir)*(psip*p + psix*x) + eer;
%           g = rhog*g(-1)+eg;
%           u = rhou*u(-1)+eu;
%           eer = er;
%end; 

% for exercise 3
 model(linear); 
           x = x(1)-(1/phi)*(r-p(1))+g;
           p = betta*p(1)+kappa*x+u;
           r = psir*r(-1)+(1-psir)*(psip*p + psix*x) + eer;
           g = rhog*g(-1)+eg;
          u = rhou*u(-1)+eu;
          eer = er;
          xx = x+u1;
          pp = p+u2;
          rr = r+u3;
end; 

% for exercise 4:
%model(linear); 
%           x = x(1)-(1/phi)*(r-p(1))+g;
%           p = betta*p(1)+kappa*x+u;
%           r = psir*r(-1)+(1-psir)*(psip*p(1) + psix*x(1)) + eer;
%           g = rhog*g(-1)+eg;
%           u = rhou*u(-1)+eu;
%           eer = er;
%end; 


% for exercise 5:
%model(linear); 
%           x = x(1)-(1/phi)*(r-p(1))+g;
%           p = betta*p(1)+kappa*x+u;
%           r = psir*r(-1)+(1-psir)*(psip*p(1) + psix*x(1)) + eer;
%           g = rhog*g(-1)+eg;
%           u = rhou*u(-1)+a12*eg+eu;
%           eer = a13*eg+er;
%end; 

%shocks;
%	var eg; stderr 0.1;
%	var eu; stderr 0.1;
%	var er; stderr 0.1;
%end;

% for exercise 3
shocks;
	var eg; stderr 0.1;
	var eu; stderr 0.1;
	var er; stderr 0.1;
	var u1; stderr 0.01;
	var u2; stderr 0.01;
	var u3; stderr 0.01;
end;
 
estimated_params;
 	phi,2.00,0,10,NORMAL_PDF,2,0.35;
%	betta,0.99,0.95,0.999,BETA_PDF,0.98,0.01;
%	kappa,2.0,0.1,3,NORMAL_PDF,2.0,0.35;
	psix,0.50,0.01,0.9,NORMAL_PDF,0.5,0.25;
	psir,0.80,0.1,0.99,BETA_PDF,0.80,0.25;
	psip,1.30,0.1,5,Normal_PDF,1.3,0.5;
%	rhog,0.80,0.1,0.99,BETA_PDF,0.80,0.25;
	rhou,0.80,0.1,0.99,BETA_PDF,0.80,0.25;
	stderr eg,0.015,0.01,1,INV_GAMMA_PDF,0.015,0.001;
	stderr eu,0.061,0.01,1,INV_GAMMA_PDF,0.061,0.0001;
	stderr er,0.161,0.01,1,INV_GAMMA_PDF,0.161,0.0001;

	%% exercise 2: less informative priors, for example
	%phi,2.00,0.1,50,NORMAL_PDF,2.0,1.0;
	%kappa,2.0,0.1,50,NORMAL_PDF,2.0,1.0;

	% for exercise 3 add
%	stderr u1,0.01,0.01,0.1,INV_GAMMA_PDF,0.01,0.0001;
%	stderr u2,0.01,0.01,0.1,INV_GAMMA_PDF,0.01,0.0001;
%	stderr u3,0.01,0.01,0.1,INV_GAMMA_PDF,0.01,0.0001;

     % for exercise 5 add
%	a12,0.1,-0.5,0.5,NORMAL_PDF,0,0.25;
%	a13,0.1,-0.5,0.5,NORMAL_PDF,0,0.25;

end;

%varobs r x p;

% for exercise 3 use instead
 varobs rr xx pp;

estimation(datafile=detdataUS_2,first_obs=117,nograph,mode_compute=6,lik_init=2,mh_replic=50000,mh_nblocks=2,mh_jscale=0.02,mh_init_scale=0.03,bayesian_irf) rr xx pp;

% for exercise 1  
% estimation(datafile=detdataUS_2,nograph,lik_init=2,mh_replic=50000,mh_nblocks=2,mh_jscale=0.02,mh_init_scale=0.03,bayesian_irf);
 
 



