% bayesian estimation of a  sticky price-sticky wage model

% The parameters are
% Theta(1);%        % intertemporal elast. of substitution 
% Theta(2);%        % probability of not adjusting price 
% Theta(3);%        % probability of not adjusting wages  
% Theta(4);%        % Coefficient on labor in u. function
% Theta(5);%        % elasticity of labor demand w.r.t. relative real wage
% Theta(6);%   		% rule of thumb parameter
% Theta(7);%        % Interest Rate Smoothing
% Theta(8);%        % Coefficents in Taylor rule: output
% Theta(9);%        %						      price inflation
% Theta(10);% % autocorrelation of technology shocks
% Theta(11);% % autocorrelation of demand shocks
% Theta(12);%  %tech. shock s.d. 
% Theta(13);%  %monetary shock s.d.
% Theta(14);%  %price shock s.d.
% Theta(15);%  %demand shock s.d

clear all;

% reading the data
[rw,i,dp,y]=textread('usadefl1d.txt','%f %f %f %f');
dy=zeros(size(dp,1),4); %data

dy(:,1)=rw; % real wages
dy(:,2)=i;  % intesest rate
dy(:,3)=dp; % inflation
dy(:,4)=y;  % output
 
% parameters of the simulation
N=10000; % number of simulations
%k=8; %number of lags

Theta_s=zeros(N,15);
loglike_s=zeros(N,1);
logpri_s=zeros(N,1);

% variance of metropolis-hasting 

variance_normal(1)=0.16;
variance_normal(2)=0.1;
variance_normal(3)=0.1;
variance_normal(4)=0.2;
variance_normal(5)=0.2;
variance_normal(6)=0;
variance_normal(7)=0.1;
variance_normal(8)=0.05;
variance_normal(9)=0.09;
variance_normal(10)=0.05;
variance_normal(11)=0.05;
variance_normal(12)=0.009;
variance_normal(13)=0.009;
variance_normal(14)=0.009;
variance_normal(15)=0.009;

variance_normal=variance_normal/20;


% Initial Values for parameters

load init.mat

% evaluating the likelihood and prior functions

[loglike]=likeliehl(dy,Theta);
logpri=priorehl(Theta);

loglikelogpri=loglike+logpri;

% save initiallikepriEHL loglikelogpri

% initial value for the acceptance rate, 

ppacc=0;
test1=0;

tic;

for j=1:N,
if j==2500
    disp('2500')
end
    if j==5000
    disp('5000')
    end
    if j==7500
    disp('7500')
    end
        newTheta=Theta+normrnd(0,variance_normal,1,15);
	    %newTheta(5)=4.33;
        newTheta(6)=0;                        %Set share of rule of thumb consumers to zero
        newlogpri=priorehl(newTheta);
        if newlogpri>-Inf
                [newloglike]=likeliehl(dy,newTheta);
                if newloglike==-Inf;
                        ratio=0;
                else
                        ratio=exp(newloglike+newlogpri-(loglike+logpri));
                        if rand<=ratio % we accept with prob ratio
                                loglike=newloglike;
                                logpri=newlogpri;
                                Theta=newTheta;
                                ppacc=ppacc+1;
                        end
                end
        end

	Theta_s(j,:)=Theta;
	loglike_s(j,1)=loglike;
	logpri_s(j,1)=logpri;

   %	ACPI=crosscorrpi(Theta,k);
   %	[ACY,CCYPI,CCYRW]=crosscorry(Theta,k);
   %	ACRW=crosscorrrw(Theta,k);
   %	ACI=crosscorri(Theta,k);
    
end
toc;

% saving acceptance rate and rate of non-solution
ppacc=ppacc/N*100;
test1=test1/N*100;

%fid4=fopen('EHLrate.txt','w');
%fprintf(fid4,'%s ','# of draws:');
%fprintf(fid4,'%d\n',N);
%fprintf(fid4,'%s ','rate of acceptance (in %):');
%fprintf(fid4,'%f\n',pau);
%fprintf(fid4,'%s ','% of indeterminancies:');
%fprintf(fid4,'%f\n',test1);   
%fclose(fid4);

disp(' Some Diagnostics ')
disp('------------------')
disp(' ')
fprintf('acceptance ratio:  %3.2f %%\n',ppacc)
disp(' ')
fprintf('number of eigenvalue restriction violations:  %3.0f (= %3.2f %% of draws )\n',test1)


save EHLoutput  % to be used in marginal.m 

% print out moments of the posterior
postmeds  = median(Theta_s,1);
postmeans = mean(Theta_s,1);
poststds  = std(Theta_s,0,1);
postmins  = min(Theta_s,[],1);
postmaxs  = max(Theta_s,[],1);
[ postmeds' poststds']


% plot the posterior densities
for p=1:15
     subplot(5,3,p);
 %        histfit(Theta_s(:,i),30);
  [f,x] = ksdensity(Theta_s(:,p));
  plot(x,f)
end

    
    